@@ -1,8 +1,12 @@
 import json
+import math
+from typing import List
+
+from opentrons.protocol_api.labware import Well
 
 from ..station import instrument_loader, labware_loader
 from ..utils import uniform_divide, WellWithVolume, MoveWithSpeed, mix_bottom_top
-from .technogenetics import StationBTechnogenetics
+from .technogenetics import StationBTechnogenetics, StationBTechnogeneticsSaliva
 from ..paired_pipette import PairedPipette
 from opentrons.types import Point
 from itertools import repeat
@@ -73,12 +77,13 @@ class StationBTechnogeneticsPairedPipette(StationBTechnogenetics):
         PairedPipette.setup(self._m300, self._m300r, self, pick_up_single=self._pick_up_single)
         super(StationBTechnogeneticsPairedPipette, self).body()
 
-    def mix_samples(self):
-        well_with_volume = WellWithVolume(self.temp_samples_m[0],
+    def mix_samples(self, wells: List[Well], stage_name: str = "mix sample"):
+        well_with_volume = WellWithVolume(wells[0],
                                           initial_vol=self._starting_vol - self._sample_mix_vol,
                                           min_height=self._sample_mix_height,
                                           headroom_height=0)
-        with PairedPipette(self._tempplate, self.temp_samples_m, start_at="mix sample") as tp:
+
+        with PairedPipette(wells[0].parent, wells, start_at=stage_name) as tp:
             tp.set_flow_rate(aspirate=self._mix_samples_rate, dispense=self._mix_samples_rate)
             tp.pick_up()
             # Custom mix_bottom_top
@@ -187,7 +192,7 @@ class StationBTechnogeneticsPairedPipette(StationBTechnogenetics):
             tp.air_gap(self._elute_air_gap)
             tp.drop_tip()
 
-    def final_transfer(self):
+    def final_transfer_movements(self):
         locs = []
         for i, (m, e) in enumerate(zip(self.mag_samples_m, self.pcr_samples_m)):
             side = -1 if i % 2 == 0 else 1
@@ -228,20 +233,33 @@ class StationBTechnogeneticsPairedPipette(StationBTechnogenetics):
         num_trans, vol_per_trans = uniform_divide(vol, self._wash_max_transfer_vol)
 
         src = []
-        mix_locs = []
         wash_locs = []
         wash_locs_index = [0, 0, 1, 1, 4, 4, 5, 5, 2, 2, 3, 3]
+        mix_locs_nowalk = []
+        mix_locs_aspirate = []
+        mix_locs_dispense_1 = []
+        mix_locs_dispense_2 = []
 
         for i, m in enumerate(self.mag_samples_m):
             wash_locs.append(source[wash_locs_index[i]])
             src.append(self.wash_getcol(i, len(self.mag_samples_m), source))
-            mix_locs.append(m.bottom(self._bottom_headroom_height).move(Point(x=2 * (-1 if i % 2 else +1))))
+            # For beads resuspension
+            mix_locs_nowalk.append(m.bottom(self._bottom_headroom_height).move(Point(x=2 * (-1 if i % 2 else +1))))
+            mix_locs_aspirate.append(m.bottom(self._resuspend_bottom_height))
+            mix_locs_dispense_1.append(m.bottom(self._resuspend_bottom_height).move(Point(x=2 * (-1 if i % 2 else +1),
+                                                                                          y=self._resuspend_y_movement)))
+            mix_locs_dispense_2.append(m.bottom(self._resuspend_bottom_height).move(Point(x=2 * (-1 if i % 2 else +1),
+                                                                                          y=-self._resuspend_y_movement)))
 
         with PairedPipette(self._magplate, self.mag_samples_m,
                            wash_locs=wash_locs,
-                           mix_locs=mix_locs,
+                           mix_locs_nowalk=mix_locs_nowalk,
+                           mix_locs_aspirate=mix_locs_aspirate,
+                           mix_locs_dispense_1=mix_locs_dispense_1,
+                           mix_locs_dispense_2=mix_locs_dispense_2,
                            start_at=wash_name) as tp:
             tp.pick_up()
+            # Add wash buffer
             for n in range(num_trans):
                 if n > 0:
                     tp.dispense(self._wash_air_gap, locationFrom="wash_locs", well_modifier="top(0)")
@@ -252,8 +270,22 @@ class StationBTechnogeneticsPairedPipette(StationBTechnogenetics):
                 if n < num_trans-1:
                     tp.air_gap(self._wash_air_gap)
 
-            # Mix
-            tp.mix(mix_reps, self._wash_mix_vol, locationFrom="mix_locs")
+            # Beads resuspension
+
+            if self._wash_mix_walk:
+                double_cycles = math.ceil(mix_reps / 2)
+                for i in range(double_cycles):
+                    tp.aspirate(self._sample_mix_vol,
+                                locationFrom="mix_locs_aspirate")
+                    tp.dispense(self._sample_mix_vol,
+                                locationFrom="mix_locs_dispense_1")
+                    tp.aspirate(self._sample_mix_vol,
+                                locationFrom="mix_locs_aspirate")
+                    tp.dispense(self._sample_mix_vol,
+                                locationFrom="mix_locs_dispense_2")
+            else:
+                tp.mix(mix_reps, self._wash_mix_vol, locationFrom="mix_locs_nowalk")
+
             tp.move_to(locationFrom="target", well_modifier="top(0)", speed=vertical_speed)
             tp.air_gap(self._wash_air_gap)
             tp.drop_tip()
@@ -264,6 +296,22 @@ class StationBTechnogeneticsPairedPipette(StationBTechnogenetics):
             self.delay(self._wait_time_wash_on, self.get_msg_format("incubate on magdeck", self.get_msg("on")))
         self.remove_supernatant(vol, stage="remove {}".format(wash_name))
 
+    def set_flow_rate(self, aspirate: float = None, dispense: float = None):
+        if aspirate is not None:
+            self._m300.flow_rate.aspirate = aspirate
+            self._m300r.flow_rate.aspirate = aspirate
+        if dispense is not None:
+            self._m300.flow_rate.dispense = dispense
+            self._m300r.flow_rate.dispense = dispense
+
+
+
+class StationBTechnogeneticsSalivaPairedPipette(StationBTechnogeneticsPairedPipette, StationBTechnogeneticsSaliva):
+    """ Build a :py:class:`.StationBTechnogeneticsSalivaPairedPipette`.
+        Everything needed should be imported from class inheritance.
+    """
+    pass
+
 if __name__ == "__main__":
     station = StationBTechnogeneticsPairedPipette(metadata={'apiLevel': '2.7'},
                                                   num_samples=96)
